<?php

class proactive_password_checker
{
    private $dictionary; //array of words loaded from dictionary file

    private $password; //password as entered by user

    public $password_words; //value of each word is equal to size of dictionary
    public $password_randoms; //value of randoms determined by get_char_score method
    public $password_substitutions; //value of sustitutions is 2
    public $password_repeats; // value of repeats is 26 * length

    public function __construct()
    {
        if(!$this->dictionary = file('cracklib-small.txt', FILE_IGNORE_NEW_LINES))
        {
            echo 'Error: failed to load dictionary, check dictionary file location and permissions.';
        }
    }

    public function load_password($password)
    {
        $this->password = $password;
        $this->find_words($this->password);
    }

    //result fetching functions
    public function get_total_score()
    {
        $total = 1;

        if(isset($this->password_words)) //words
        {
            $total *= pow(sizeof($this->dictionary), sizeof($this->password_words));
        }

        if(isset($this->password_randoms)) //random charactors
        {
            for($i=0;$i<sizeof($this->password_randoms);$i++)
            {
                $total *= $this->get_char_score($this->password_randoms[$i]);
            }
        }

        if(isset($this->password_substitutions)) //substituted charactors
        {
            $total *= pow(2, sizeof($this->password_substitutions));
        }

        if(isset($this->password_repeats)) //repeated charactors
        {
            for($i=0;$i<sizeof($this->password_repeats);$i++)
            {
                $total *= $this->get_char_score($this->password_repeats[$i][0]) * pow(2, strlen($this->password_repeats[$i])-1);
            }
        }
        return $total;
    }

    public function strength_guage()
    {
        return strlen($this->get_total_score());
    }

    private function get_char_score($char)
    {
        if(ctype_digit($char))      return 10;
        else if(ctype_lower($char)) return 26;
        else if(ctype_upper($char)) return 52;
        else if(ctype_punct($char)) return 44;
        else return 1;//unknown char, return 1 to avoid zero result.
    }

    private function find_words($test_string)
    {
        $longest_word_index = false;//index of the word in the dictionary
        $longest_word_pos = false;//position of the word in the test sting

        $norm_test_string = $this->normalise_string($test_string);

        for($i=0;$i<sizeof($this->dictionary);$i++)
        {
            $string_pos = strpos($norm_test_string, $this->dictionary[$i]);

            if($string_pos !== false)
            {
                if(strlen($this->dictionary[$i]) > strlen($this->dictionary[$longest_word_index]))
                {
                    $longest_word_index = $i;
                    $longest_word_pos = $string_pos;
                }
            }
        }

        if(strlen($this->dictionary[$longest_word_index]) < 4)
            $longest_word_index = false; // ignore really short words of 3 chars or less

        if($longest_word_index) //if a word has been found
        {
            //add word to password elements
            $this->password_words[] = $this->dictionary[$longest_word_index];
            //add score to tally

            $string_a_start = 0;
            $string_a_length = $longest_word_pos;
            $string_b_start = $string_a_length + strlen($this->dictionary[$longest_word_index]);
            $string_b_length = strlen($norm_test_string) - $string_b_start;

            $string_a = substr($test_string, $string_a_start, $string_a_length);//cut anything before the word
            $string_b = substr($test_string, $string_b_start, $string_b_length);//cut anything after the word
            
            $word_section = substr($test_string, $string_a_length, strlen($this->dictionary[$longest_word_index]));//cut the word

            //check for charactor substitutions
            $this->find_character_subs($word_section);

            if(isset($string_a))
                $this->find_words($string_a);

            if(isset($string_b))
                $this->find_words($string_b);
        }
        else //look for repeats and randoms
        {
            $this->find_repeats_and_randoms($test_string);
        }
    }

    private function find_repeats_and_randoms($str)//now working
    {
        $i=0;
        for($i=0;$i<strlen($str)-1;)//traverse the string
        {
            if($str[$i] == $str[$i+1]) // check if current char is equal to next char
            {
                $temp = '';//clear first

            	do
            	{
					$temp[] = $str[$i++];
            	}
            	while($i < strlen($str) && $str[$i-1] == $str[$i]);

            	$this->password_repeats[] = implode($temp);
            }
            else //current char does not match previous, assume random
            {
                $this->password_randoms[] = $str[$i++];
            }
        }

        //catch last char, will be random if it hasen't already been caught
        if(isset($str[$i]))
        {
            $this->password_randoms[] = $str[$i];
        }
    }

    private function find_character_subs($test_string)
    {
        $norm_test_string = $this->normalise_string($test_string);

        for($i = 0; $i < strlen($test_string); $i++)
        {
            if($test_string[$i] != $norm_test_string[$i])
            {
                $this->password_substitutions[] = $test_string[$i];
            }
        }
    }

    private function normalise_string($submitted_string)
    {
        $string = $submitted_string;

        for($i=0;$i<strlen($submitted_string);$i++)
        {
            switch($submitted_string[$i])
            {
                case '@' : $string[$i] = 'a';
                break;
                case '8' : $string[$i] = 'b';
                break;
                case '(' : $string[$i] = 'c';
                break;
                case '6' : $string[$i] = 'd';
                break;
                case '3' : $string[$i] = 'e';
                break;
                case '9' : $string[$i] = 'g';
                break;
                case '#' : $string[$i] = 'h';
                break;
                case '1' : $string[$i] = 'i';
                break;
                case '!' : $string[$i] = 'i';
                break;
                case '<' : $string[$i] = 'k';
                break;
                case '0' : $string[$i] = 'o';
                break;
                case '5' : $string[$i] = 's';
                break;
                case '$' : $string[$i] = 's';
                break;
                case '+' : $string[$i] = 't';
                break;
                case '>' : $string[$i] = 'v';
                break;
                case '%' : $string[$i] = 'x';
                break;
                case '?' : $string[$i] = 'y';
                break;
            }

    //a=@ b=8 c=( d=6 e=3 f=# g=9 h=# i=1 i=! k=< l=1
    //l=i o=0 q=9 s=5 s=$ t=+ v=> v=< w=uu    w=2u    x=% y=?
        }
        return strtolower($string);
    }
}
?>